@@ -915,10 +915,10 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser, const char *url, c
 			continue;
 		}
 
-		if (strstr (file, "://") != NULL
-				|| file[0] == G_DIR_SEPARATOR) {
-			totem_pl_parser_add_one_url_ext (parser,
-					file, title, genre);
+		if (strstr (file, "://") != NULL || file[0] == G_DIR_SEPARATOR) {
+			if (totem_pl_parser_parse (parser, file, FALSE) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
+				totem_pl_parser_add_one_url_ext (parser, file, title, genre);
+			}
 		} else {
 			char *uri, *base, *escaped;
 
@@ -928,8 +928,9 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser, const char *url, c
 
 			uri = g_strdup_printf ("%s/%s", base, escaped);
 
-			totem_pl_parser_add_one_url_ext (parser,
-					uri, title, genre);
+			if (totem_pl_parser_parse (parser, uri, FALSE) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
+				totem_pl_parser_add_one_url_ext (parser, file, title, genre);
+			}
 
 			g_free (escaped);
 			g_free (uri);
@@ -1371,11 +1372,15 @@ totem_pl_parser_add_smil (TotemPlParser *parser, const char *url, gpointer data)
 static TotemPlParserResult
 totem_pl_parser_add_asf (TotemPlParser *parser, const char *url, gpointer data)
 {
-	if (data != NULL &&
-		(g_str_has_prefix (data, "[Reference]") == FALSE
+	if (data == NULL) {
+		totem_pl_parser_add_one_url (parser, url, NULL);
+		return TOTEM_PL_PARSER_RESULT_SUCCESS;
+	}
+
+	if (g_str_has_prefix (data, "[Reference]") == FALSE
 		 && g_ascii_strncasecmp (data, "<ASX", strlen ("<ASX")) != 0
 		 && strstr (data, "<ASX") == NULL
-		 && strstr (data, "<asx") == NULL)
+		 && strstr (data, "<asx") == NULL
 		 && g_str_has_prefix (data, "ASF ") == FALSE) {
 		totem_pl_parser_add_one_url (parser, url, NULL);
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
@@ -1650,7 +1655,7 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 	const char *mimetype;
 	guint i;
 	gpointer data = NULL;
-	TotemPlParserResult ret = FALSE;
+	TotemPlParserResult ret = TOTEM_PL_PARSER_RESULT_ERROR;
 	char *super;
 	gboolean found;
 
@@ -1702,8 +1707,11 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 
 	parser->priv->recurse_level--;
 
+	if (ret == TOTEM_PL_PARSER_RESULT_SUCCESS)
+		return ret;
+
 	super = gnome_vfs_get_supertype_from_mime_type (mimetype);
-	for (i = 0; i < G_N_ELEMENTS (ignore_types); i++) {
+	for (i = 0; i < G_N_ELEMENTS (ignore_types) && super != NULL; i++) {
 		if (gnome_vfs_mime_type_is_supertype (ignore_types[i].mimetype) != FALSE) {
 			if (strcmp (super, ignore_types[i].mimetype) == 0) {
 				g_free (super);
