@@ -52,6 +52,7 @@ struct TotemPlaylistClass {
 	GtkVBoxClass parent_class;
 
 	void (*changed) (TotemPlaylist *playlist);
+	void (*item_activated) (TotemPlaylist *playlist);
 	void (*active_name_changed) (TotemPlaylist *playlist);
 	void (*current_removed) (TotemPlaylist *playlist);
 	void (*repeat_toggled) (TotemPlaylist *playlist, gboolean repeat);
@@ -88,8 +89,6 @@ gboolean   totem_playlist_get_current_metadata (TotemPlaylist *playlist,
 						char         **album);
 gboolean   totem_playlist_set_title (TotemPlaylist *playlist,
 				     const char *title);
-void       totem_playlist_set_cdindex (TotemPlaylist *playlist,
-				       const char *cdindex_id);
 
 #define    totem_playlist_has_direction(playlist, direction) (direction == TOTEM_PLAYLIST_DIRECTION_NEXT ? totem_playlist_has_next_mrl (playlist) : totem_playlist_has_previous_mrl (playlist))
 gboolean   totem_playlist_has_previous_mrl (TotemPlaylist *playlist);
