@@ -225,6 +225,7 @@ totem_action_menu_popup (Totem *totem, guint button)
 			"totem_right_click_menu");
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
 			button, gtk_get_current_event_time ());
+	gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
 }
 
 static gboolean
@@ -538,6 +539,13 @@ totem_action_pause (Totem *totem)
 	}
 }
 
+static void
+totem_action_set_cursor (Totem *totem, gboolean state)
+{
+	totem->cursor_shown = state;
+	bacon_video_widget_set_show_cursor (totem->bvw, state);
+}
+
 static gboolean
 window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
 		Totem *totem)
@@ -550,7 +558,7 @@ window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
 		totem_action_save_size (totem);
 		update_fullscreen_size (totem);
 		bacon_video_widget_set_fullscreen (totem->bvw, TRUE);
-		bacon_video_widget_set_show_cursor (totem->bvw, FALSE);
+		totem_action_set_cursor (totem, FALSE);
 
 		if (bacon_video_widget_is_playing (totem->bvw) != FALSE)
 			totem_scrsaver_disable (totem->scr);
@@ -562,7 +570,7 @@ window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
 
 		popup_hide (totem);
 		bacon_video_widget_set_fullscreen (totem->bvw, FALSE);
-		bacon_video_widget_set_show_cursor (totem->bvw, TRUE);
+		totem_action_set_cursor (totem, TRUE);
 
 		totem_scrsaver_enable (totem->scr);
 
@@ -765,8 +773,10 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		}
 
 		subtitle_uri = totem_uri_get_subtitle_uri (mrl);
+		totem_gdk_window_set_waiting_cursor (totem->win->window);
 		retval = bacon_video_widget_open_with_subtitle (totem->bvw,
 				mrl, subtitle_uri, &err);
+		gdk_window_set_cursor (totem->win->window, NULL);
 		totem->mrl = g_strdup (mrl);
 
 		/* Play/Pause */
@@ -1192,7 +1202,9 @@ on_got_redirect (BaconVideoWidget *bvw, const char *mrl, Totem *totem)
 	g_message ("on_got_redirect %s", mrl);
 	//FIXME we need to check for relative paths here
 	bacon_video_widget_close (totem->bvw);
+	totem_gdk_window_set_waiting_cursor (totem->win->window);
 	bacon_video_widget_open (totem->bvw, mrl, NULL);
+	gdk_window_set_cursor (totem->win->window, NULL);
 	bacon_video_widget_play (bvw, NULL);
 }
 
@@ -2468,7 +2480,7 @@ popup_hide (Totem *totem)
 
 	popup_timeout_remove (totem);
 
-	bacon_video_widget_set_show_cursor (totem->bvw, FALSE);
+	totem_action_set_cursor (totem, FALSE);
 
 	return FALSE;
 }
@@ -2504,7 +2516,7 @@ on_video_motion_notify_event (GtkWidget *widget, GdkEventMotion *event,
 
 	gtk_widget_show_all (totem->exit_popup);
 	gtk_widget_show_all (totem->control_popup);
-	bacon_video_widget_set_show_cursor (totem->bvw, TRUE);
+	totem_action_set_cursor (totem, TRUE);
 
 	popup_timeout_add (totem);
 	totem->popup_in_progress = FALSE;
@@ -2553,7 +2565,29 @@ on_eos_event (GtkWidget *widget, Totem *totem)
 }
 
 static gboolean
-totem_action_handle_key (Totem *totem, GdkEventKey *event)
+totem_action_handle_key_release (Totem *totem, GdkEventKey *event)
+{
+	gboolean retval = TRUE;
+
+	/* Alphabetical */
+	switch (event->keyval) {
+	case GDK_Left:
+	case GDK_Right:
+		totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), FALSE);
+		if (bacon_video_widget_can_direct_seek (totem->bvw)) {
+			if (totem->was_playing) {
+				bacon_video_widget_play (totem->bvw, NULL);
+			}
+			totem->seek_in_progress = FALSE;
+		}
+		break;
+	}
+
+	return retval;
+}
+
+static gboolean
+totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 {
 	gboolean retval = TRUE;
 
@@ -2616,9 +2650,6 @@ totem_action_handle_key (Totem *totem, GdkEventKey *event)
 	case GDK_m:
 		bacon_video_widget_dvd_event (totem->bvw, BVW_DVD_ROOT_MENU);
 		break;
-	case GDK_Menu:
-		totem_action_menu_popup (totem, 0);
-		break;
 #ifdef HAVE_XFREE
 	case XF86XK_AudioNext:
 #endif /* HAVE_XFREE */
@@ -2659,8 +2690,16 @@ totem_action_handle_key (Totem *totem, GdkEventKey *event)
 		totem_action_fullscreen (totem, FALSE);
 		break;
 	case GDK_Left:
-		if (event->state & GDK_SHIFT_MASK)
-		{
+		totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), TRUE);
+		if (bacon_video_widget_can_direct_seek (totem->bvw)) {
+			if (!totem->seek_in_progress) {
+				totem->was_playing = bacon_video_widget_is_playing (totem->bvw);
+				bacon_video_widget_pause (totem->bvw);
+				/* We need to remember that we are already paused and seeking */
+				totem->seek_in_progress = TRUE;
+			}
+		}
+		if (event->state & GDK_SHIFT_MASK) {
 			totem_action_seek_relative (totem,
 					SEEK_BACKWARD_SHORT_OFFSET);
 		} else if (event->state & GDK_CONTROL_MASK) {
@@ -2672,15 +2711,24 @@ totem_action_handle_key (Totem *totem, GdkEventKey *event)
 		}
 		break;
 	case GDK_Right:
-		if (event->state & GDK_SHIFT_MASK)
-		{
+		totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), TRUE);
+		if (bacon_video_widget_can_direct_seek (totem->bvw)) {
+			if (!totem->seek_in_progress) {
+				totem->was_playing = bacon_video_widget_is_playing (totem->bvw);
+				bacon_video_widget_pause (totem->bvw);
+				/* We need to remember that we are already paused and seeking */
+				totem->seek_in_progress = TRUE;
+			}
+		}
+		if (event->state & GDK_SHIFT_MASK) {
 			totem_action_seek_relative (totem,
 					SEEK_FORWARD_SHORT_OFFSET);
 		} else if (event->state & GDK_CONTROL_MASK) {
 			totem_action_seek_relative (totem,
 					SEEK_FORWARD_LONG_OFFSET);
 		} else {
-			totem_action_seek_relative (totem, SEEK_FORWARD_OFFSET);
+			totem_action_seek_relative (totem,
+					SEEK_FORWARD_OFFSET);
 		}
 		break;
 	case GDK_space:
@@ -2705,11 +2753,21 @@ totem_action_handle_key (Totem *totem, GdkEventKey *event)
 	case GDK_2:
 		totem_action_set_scale_ratio (totem, 2);
 		break;
+	case GDK_Menu:
 	case GDK_F10:
-		if (event->state & GDK_SHIFT_MASK)
-			totem_action_menu_popup (totem, 0);
-		else
-			retval = FALSE;
+		/* Chain up if the playlist has the focuse */
+		{
+			GtkWidget *focused;
+
+			if (event->keyval != GDK_Menu && !(event->state & GDK_SHIFT_MASK))
+				return FALSE;
+
+			focused = gtk_window_get_focus (GTK_WINDOW (totem->win));
+			if (focused != NULL && gtk_widget_is_ancestor (focused, GTK_WIDGET (totem->playlist)) != FALSE)
+				return FALSE;
+		}
+
+		totem_action_menu_popup (totem, 0);
 		break;
 	default:
 		retval = FALSE;
@@ -2781,7 +2839,12 @@ on_window_key_press_event (GtkWidget *win, GdkEventKey *event, Totem *totem)
 		case GDK_s:
 		case GDK_Right:
 		case GDK_Left:
-			return totem_action_handle_key (totem, event);
+			if (event->type == GDK_KEY_PRESS) {
+				return totem_action_handle_key_press (totem, event);
+			}
+			else {
+				return totem_action_handle_key_release (totem, event);
+			}
 	}
 
 	/* If we have modifiers, and either Ctrl, Mod1 (Alt), or any
@@ -2795,7 +2858,12 @@ on_window_key_press_event (GtkWidget *win, GdkEventKey *event, Totem *totem)
 			|| (event->state & GDK_MOD5_MASK)))
 		return FALSE;
 
-	return totem_action_handle_key (totem, event);
+	if (event->type == GDK_KEY_PRESS) {
+		return totem_action_handle_key_press (totem, event);
+	}
+	else {
+		return totem_action_handle_key_release (totem, event);
+	}
 }
 
 static int
@@ -3130,9 +3198,11 @@ totem_callback_connect (Totem *totem)
 			G_CALLBACK (vol_slider_released_cb), totem);
 
 	/* Connect the keys */
-	gtk_widget_add_events (totem->win, GDK_KEY_PRESS_MASK);
+	gtk_widget_add_events (totem->win, GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
 	g_signal_connect (G_OBJECT(totem->win), "key_press_event",
 			G_CALLBACK (on_window_key_press_event), totem);
+	g_signal_connect (G_OBJECT(totem->win), "key_release_event",
+			G_CALLBACK (on_window_key_press_event), totem);
 
 	/* Connect the mouse wheel */
 	gtk_widget_add_events (totem->win, GDK_SCROLL_MASK);
@@ -3321,9 +3391,12 @@ video_widget_create (Totem *totem)
 			GTK_WIDGET (totem->bvw));
 
 	/* Events for the widget video window as well */
-	gtk_widget_add_events (GTK_WIDGET (totem->bvw), GDK_KEY_PRESS_MASK);
+	gtk_widget_add_events (GTK_WIDGET (totem->bvw),
+			GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
 	g_signal_connect (G_OBJECT(totem->bvw), "key_press_event",
 			G_CALLBACK (on_window_key_press_event), totem);
+	g_signal_connect (G_OBJECT(totem->bvw), "key_release_event",
+			G_CALLBACK (on_window_key_press_event), totem);
 
 	g_signal_connect (G_OBJECT (totem->bvw), "drag_data_received",
 			G_CALLBACK (drop_video_cb), totem);
@@ -3449,6 +3522,7 @@ main (int argc, char **argv)
 	/* Init totem itself */
 	totem->prev_volume = -1;
 	totem->gc = gc;
+	totem->cursor_shown = TRUE;
 
 	/* Main window */
 	totem->xml = totem_interface_load ("totem.glade", _("main window"),
@@ -3515,6 +3589,7 @@ main (int argc, char **argv)
 	/* Show ! gtk_main_iteration trickery to show all the widgets
 	 * we have so far */
 	gtk_widget_show (totem->win);
+	totem_gdk_window_set_waiting_cursor (totem->win->window);
 	update_fullscreen_size (totem);
 	long_action ();
 
@@ -3536,6 +3611,7 @@ main (int argc, char **argv)
 	} else {
 		totem_action_set_mrl (totem, NULL);
 	}
+	gdk_window_set_cursor (totem->win->window, NULL);
 
 	if (bacon_message_connection_get_is_server (totem->conn) != FALSE)
 	{
